import 'package:flutter/foundation.dart';
import 'package:tap_network_kit/src/exceptions/illegal_state_exception.dart';

enum SupportedLocales { AR, EN, FR }

mixin AppInfo {
  //auth information for headers
  static String _authToken;
  static String _appName;
  static String _appId;
  static String _device;
  static String _deviceId;
  static String _appVersion;
  static Map<Object, Object> _applicationInfo;
  static String _localeString = 'en';

  static String _requirer;
  static String _requirerOS;
  static String _requirerOSVersion;

  static StringBuffer _appInfoString;

  /// Sets auth token.
  /// @param context   the context
  /// @param authToken the auth token
  static void setAuthToken(
      String authToken, String appName, String appId, String appVersion) {
    AppInfo._authToken = authToken;
    AppInfo._appName = appName;
    AppInfo._appId = appId;
    AppInfo._appVersion = appVersion;
  }

  ///
  /// Gets auth token.
  /// @return the auth token

  static String getAuthToken() {
    return _authToken;
  }

  /// Sets device.
  /// @param String   device
  static void setDevice(String device) {
    AppInfo._device = device;
  }

  /// Sets device id.
  /// @param String   deviceId
  static void setDeviceID(String deviceId) {
    AppInfo._deviceId = deviceId;
  }

    /// Sets requirer OS Version.
  /// @param String   version
  static void setRequirerOSVersion(String version) {
    AppInfo._requirerOSVersion = version;
  }


  /// Sets requirer.
  /// @param String   requirer
  static void setRequirer(String requirer) {
    AppInfo._requirer = requirer;
  }

  /// Sets requirer.
  /// @param String   requirer OS
  static void setRequirerOS(String requirerOS) {
    AppInfo._requirerOS = requirerOS;
  }
  

  ///
  /// Sets locale.
  /// @param locale the locale

  static void setLocale(String locale) {
    AppInfo._localeString = locale.length < 2 ? locale : locale.substring(0, 2);
  }

  /// Gets locale string.
  /// @return the locale string

  static String getLocaleString() {
    return _localeString;
  }

  // @param localeString the locale string
  // @return the supported locales
  static String findByString(String localeString) {
    for (SupportedLocales locale in SupportedLocales.values) {
      if (describeEnum(locale).toString().toLowerCase() ==
          localeString.toLowerCase()) {
        return describeEnum(locale).toString();
      }
    }
    return describeEnum(SupportedLocales.EN).toString();
  }

  ///initialize application using its id.
  /// @param applicationId
  /// @return _applicationInfo map.
  static void initApplicationInfo() {
    _applicationInfo = <String, String>{};
    _applicationInfo.putIfAbsent('app_id', () => _appId);
    _applicationInfo.putIfAbsent('app_name', () => _appName);
    _applicationInfo.putIfAbsent('app_version', () => _appVersion);
    _applicationInfo.putIfAbsent('requirer', () => _requirer);
    _applicationInfo.putIfAbsent('requirer_os', () => _requirerOS);
    _applicationInfo.putIfAbsent(
        'requirer_os_version', () => _requirerOSVersion);
    _applicationInfo.putIfAbsent(
        'app_locale', () => findByString(AppInfo.getLocaleString()).toString());
  }

  ///
  ///Gets application info.
  ///@return the application info

  static String getApplicationInfo() {
    _appInfoString ??= StringBuffer();
    if (_applicationInfo == null)
      throw TapIllegalStateException(
          'Application Info can not be null', AppInfo.classInfo());
    else {
      _applicationInfo.forEach((Object k, Object v) {
        _appInfoString.write(k);
        _appInfoString.write('=');
        _appInfoString.write(v);
        _appInfoString.write('|');
      });
      return _appInfoString.length != 0
          ? _appInfoString.toString().substring(0, _appInfoString.length - 1)
          : _appInfoString.toString();
    }
  }

  static String classInfo() {
    return 'package:tap_network_kit/src/app_info/AppInfo.class';
  }
}
