import 'package:chopper/chopper.dart';
import 'package:flutter/foundation.dart';
import 'package:logging/logging.dart';
import 'package:tap_network_kit/src/constants/api_constants.dart';
import 'package:tap_network_kit/src/converter/error_converter.dart';
import 'package:tap_network_kit/src/converter/tap_converter.dart';
import 'package:tap_network_kit/src/interceptors/request_interceptor.dart';
import 'package:tap_network_kit/src/interface/response_model_interface.dart';

/// Created by h.shestawy on "2019-07-16".
/// Copyright © "2019" Tap Payments. All rights reserved.

mixin RetrofitHelper {
  /// aray of request interceptors
  static Iterable<RequestInterceptor> _interceptors;

  /// get array of request interceptors
  /// @return    Iterable<RequestInterceptor> [HeaderInterceptors - LoggingInterceptors]
  static Iterable<RequestInterceptor> getInterceptors() {
    return _interceptors ??= <RequestInterceptor>[
      getHeadersInterceptors(),
      getLoggingInterceptors(),
      TapRequestInterceptor(),
    ];
  }

  /// @param      JSONModelInterface client can pass any Model that implements JSONModelInterface
  /// @return     TapConverter [ Serialize request - deserialize response to specific model - deserialize error to generic error]
  static Converter getConverters(ResponseModelInterface jsonModel) {
    return TapConverter<ResponseModelInterface>(jsonModel);
  }

  /// prepare headers interceptor
  /// @return    HeadersInterceptor populated with common headers
  static HeadersInterceptor getHeadersInterceptors() {
    /// for web, only keep this header
    ///
    if (kIsWeb)
      return const HeadersInterceptor(<String, String>{
        APIConstanst.CONTENT_TYPE_KEY: APIConstanst.CONTENT_TYPE_VALUE,
      });
    return const HeadersInterceptor(<String, String>{
      APIConstanst.CONTENT_TYPE_KEY: APIConstanst.CONTENT_TYPE_VALUE,
      APIConstanst.CACHE_CONTROL_KEY: APIConstanst.CACHE_CONTROL_VALUE,
      APIConstanst.AUTH_TOKEN_KEY: APIConstanst.AUTH_TOKEN_VALUE,
      APIConstanst.APP_VERSION: APIConstanst.APP_VERSION_VALUE,
      APIConstanst.APP_LICENSE_CODE: APIConstanst.APP_LICENSE_CODE_VALUE,
      APIConstanst.LANG_CODE: APIConstanst.LANG_CODE_VALUE,
      APIConstanst.ACCEPT_CHARSET_HEADER_KEY: APIConstanst.ACCEPT_CHARSET_HEADER_VALUE,
      APIConstanst.CHARSET_HEADER_KEY: APIConstanst.CHARSET_HEADER_VALUE,
      // APIConstanst.APPLICATION: AppInfo.getApplicationInfo()
    });
  }

  /// prepare logging interceptor
  /// @return    LoggingInterceptor
  static HttpLoggingInterceptor getLoggingInterceptors() {
    Logger.root.level = Level.ALL;
    Logger.root.onRecord.listen((dynamic rec) {
      print('${rec.level.name} : ${rec.time} : ${rec.message}');
    });
    return HttpLoggingInterceptor();
  }

  /// prepare error converter
  /// @return    TapErrorConverter
  static ErrorConverter getErrorConverter() {
    return TapErrorConverter();
  }
}
